@@ -1,24 +1,88 @@
 
-;; Copyright (C) 2008-2013 Tommi Höynälänmaa
+;; Copyright (C) 2008-2019 Tommi Höynälänmaa
 ;; Distributed under GNU Lesser General Public License version 3,
 ;; see file doc/LGPL-3.
 
 
-(import (guile))
+(define-module (theme-d runtime theme-d-stdlib-support)
+  #:export (_b_raise
+	    theme-exit 
+	    ;; theme-command-line
+	    theme-debug-print
+	    theme-enable-rte-exception-info
+	    theme-disable-rte-exception-info
+	    theme-d-condition-kind1
+	    theme-d-condition-info1
+	    _i_vector-ref
+	    _i_mutable-vector-ref
+	    _i_mutable-vector-set!
+	    _i_value-vector-ref
+	    _i_mutable-value-vector-ref
+	    _i_mutable-value-vector-set!
+	    _b_vector-length
+	    theme-string-match
+	    integer->real
+	    real->integer
+	    theme-round
+	    theme-truncate
+	    theme-floor
+	    theme-ceiling
+	    xor
+	    theme-real-integer/
+	    real-to-rational-impl
+	    theme-open-output-file
+	    theme-open-input-file
+	    theme-close-output-port
+	    theme-close-input-port
+	    theme-prim-write
+	    theme-prim-display
+	    theme-newline
+	    theme-read-character
+	    theme-peek-character
+	    theme-read
+	    theme-char-ready?
+	    theme-current-output-port
+	    theme-current-error-port
+	    theme-current-input-port
+	    theme-call-with-input-string
+	    theme-call-with-output-string
+	    theme-prim-console-display
+	    theme-console-newline
+	    theme-console-read-character
+	    theme-console-read
+	    theme-console-char-ready?
+	    theme-getenv
+	    theme-delete-file
+	    d-car
+	    d-cdr
+	    reverse-search-goops-class
+	    reverse-get-goops-class
+	    make-hash-table-with-size
+	    theme-hashx-ref
+	    theme-hashx-exists?
+	    theme-hashx-set!
+	    theme-hashx-remove!
+	    hash-count-elements
+	    theme-string-assoc
+	    theme-hash-for-each
+	    hash-contents
+	    theme-string-hash
+	    theme-string->integer
+	    theme-string->real
+	    theme-string->real-number
+	    theme-call/cc
+	    theme-call/cc-without-result
+	    theme-with-exception-handler))
+
+
+(import (except (rnrs) assert map))
 (import (srfi srfi-1))
-(import (srfi srfi-13))
-(import (rnrs exceptions))
-(import	(rnrs conditions))
-(import (rnrs io ports))
-(import (rnrs arithmetic bitwise))
-(import (only (rnrs base) integer-valued?))
 (import (ice-9 regex))
 (import (oop goops))
-(import (statprof))
 
 (import (th-scheme-utilities stdutils))
-(import (theme-d runtime runtime0))
-
+(import (theme-d runtime runtime-theme-d-environment))
+(import (theme-d runtime theme-d-support))
 
 (define (raise-simple s-kind)
   (raise (make-theme-d-condition s-kind '())))
@@ -44,8 +108,8 @@
 				 (list (cons 'i-exit-code i-exit-code)))))
 
 
-(define (theme-command-line)
-  gl-l-command-line)
+;; (define (theme-command-line)
+;;   gl-l-command-line)
 
 
 (define (theme-debug-print x)
@@ -84,12 +148,6 @@
   (vector-set! vec (+ i 1) item))
 
 
-(define (_i_value-vector-ref vec i)
-  (vector-ref vec (+ i 1)))
-
-
-
-
 (define (_i_value-vector-ref vec i)
   (vector-ref vec (+ i 1)))
 
@@ -110,11 +168,15 @@
 
 
 (define (theme-string-match str-pattern str-source)
-  (let ((v-result (string-match str-pattern str-source)))
-    (if (vector? v-result)
-	(let ((pr-ind (vector-ref v-result 1)))
-	  (list (vector-ref v-result 0) (car pr-ind) (cdr pr-ind)))
-        '())))
+  (let ((match-result (string-match str-pattern str-source)))
+    (if (regexp-match? match-result)
+	(list
+	 (match:substring match-result)
+	 (match:start match-result)
+	 (match:end match-result))
+	(begin
+	  (assert (eq? match-result #f))
+	  '()))))
 
 
 (define (integer->real n)
@@ -162,6 +224,40 @@
       (/ r n)))
 
 
+(define (theme-call/cc normal-type jump-type body-proc)
+  (let* ((proc-type
+	  (_i_make-procedure-type (_i_make-tuple-type jump-type)
+				  _b_none #t #f #t #f #t))
+	 (result
+	  (call/cc
+	   (lambda (proc)
+	     (let ((my-proc (_i_make-procedure proc-type proc)))
+	       (_i_call-proc body-proc (list my-proc) (list proc-type)))))))
+    result))
+
+
+(define (theme-call/cc-without-result body-proc)
+  (let* ((proc-type
+	  (_i_make-procedure-type (_i_make-tuple-type)
+				  _b_none #t #f #t #f #t))
+	 (result
+	  (call/cc
+	   (lambda (proc)
+	     (let* ((actual-proc (lambda () (proc '())))
+		    (my-proc (_i_make-procedure proc-type
+						actual-proc)))
+	       (_i_call-proc body-proc (list my-proc) (list proc-type)))))))
+    result))
+
+
+(define (theme-with-exception-handler type-result proc-handler proc-body)
+  (with-exception-handler
+   (lambda (variable)
+     (_i_call-proc proc-handler (list variable) (list _b_<object>)))
+   (lambda ()
+     (_i_call-proc proc-body '() '()))))
+
+
 ;; *** (standard-library math) ***
 
 
@@ -184,7 +280,7 @@
   (guard (exc (else
   	       (raise (make-file-exception
   		       'error-opening-output-file filename))))
-  	 (open-output-file filename)))
+  	 (open-file-output-port filename (file-options no-fail))))
 
 
 (define (theme-open-input-file filename)
@@ -210,6 +306,14 @@
   	 (close-input-port ip)))
 
 
+(define (theme-prim-write obj op)
+  (guard (exc (else
+  	       (raise (make-file-exception
+  		       'error-writing-object
+		       (i/o-error-filename exc)))))
+  	 (write obj op)))
+
+
 (define (theme-prim-display obj op)
   (guard (exc (else
   	       (raise (make-file-exception
@@ -309,7 +413,7 @@
   	 (char-ready? ip)))
 
 
-;; Filename makes probably no sense in the following two procedures.
+;; Filename makes probably no sense in the following three procedures.
 (define (theme-current-output-port)
   (guard (exc (else
   	       (raise (make-file-exception
@@ -317,6 +421,13 @@
   	 (current-output-port)))
 
 
+(define (theme-current-error-port)
+  (guard (exc (else
+  	       (raise (make-file-exception
+  		       'current-error-port:runtime-error ""))))
+  	 (current-error-port)))
+
+
 (define (theme-current-input-port)
   (guard (exc (else
   	       (raise (make-file-exception
@@ -334,9 +445,8 @@
     result))
 
 
-(define (theme-call-with-output-string str proc)
+(define (theme-call-with-output-string proc)
   (call-with-output-string
-   str
    (lambda (port)
      (_i_call-proc proc (list port) (list (theme-class-of port))))))
 
@@ -432,14 +542,12 @@
   (let* ((raw-proc-hash
 	  (lambda lst-args
 	    (_i_call-proc proc-hash lst-args
-			  (list _b_<object> _b_<integer>))))
+			   (list _b_<object> _b_<integer>))))
 	 (raw-proc-assoc
 	  (lambda lst-args
-	    (let ((res
-		   (_i_call-proc proc-assoc lst-args
-				 ;; Maybe the second type should be more exact.
-				 (list _b_<object> _b_<object>))))
-	      (if (null? res) #f res))))
+	    (_i_call-proc proc-assoc lst-args
+			   ;; Maybe the second type should be more exact.
+			   (list _b_<object> _b_<object>))))
 	 (o-res
 	  (hashx-ref raw-proc-hash raw-proc-assoc hashtable obj-key
 		     obj-default)))
@@ -449,65 +557,47 @@
   (let* ((raw-proc-hash
 	  (lambda lst-args
 	    (_i_call-proc proc-hash lst-args
-			  (list _b_<object> _b_<integer>))))
+			   (list _b_<object> _b_<integer>))))
 	 (raw-proc-assoc
 	  (lambda lst-args
-	    (let ((res
-		   (_i_call-proc proc-assoc lst-args
-				 ;; Maybe the second type should be more exact.
-				 (list _b_<object> _b_<object>))))
-	      (if (null? res) #f res))))
+	    (_i_call-proc proc-assoc lst-args
+			   ;; Maybe the second type should be more exact.
+			   (list _b_<object> _b_<object>))))
 	 (o-res
 	  (hashx-ref raw-proc-hash raw-proc-assoc hashtable obj-key '())))
     (not-null? o-res)))
 
 (define (theme-hashx-set! proc-hash proc-assoc hashtable obj-key obj-value)
-  (let ((raw-proc-hash
-	 (lambda lst-args
-	   (let ((result
-		  (_i_call-proc proc-hash lst-args
-				(list _b_<object> _b_<integer>))))
-	     result)))
-	(raw-proc-assoc
-	 (lambda lst-args
-	   (let ((res
-		  (_i_call-proc proc-assoc lst-args
-				;; Maybe the second type should be more exact.
-				(list _b_<object> _b_<object>))))
-	     (if (null? res) #f res)))))
+  (let* ((raw-proc-hash
+	  (lambda lst-args
+	    (_i_call-proc proc-hash lst-args
+			   (list _b_<object> _b_<integer>))))
+	 (raw-proc-assoc
+	  (lambda lst-args
+	    (_i_call-proc proc-assoc lst-args
+			   ;; Maybe the second type should be more exact.
+			   (list _b_<object> _b_<object>)))))
     (hashx-set! raw-proc-hash raw-proc-assoc hashtable obj-key obj-value)))
 
 (define (theme-hashx-remove! proc-hash proc-assoc hashtable obj-key)
-  (let ((raw-proc-hash
-	 (lambda lst-args
-	   (let ((result
-		  (_i_call-proc proc-hash lst-args
-				(list _b_<object> _b_<integer>))))
-	     result)))
-	(raw-proc-assoc
-	 (lambda lst-args
-	   (let ((res
-		  (_i_call-proc proc-assoc lst-args
-				;; Maybe the second type should be more exact.
-				(list _b_<object> _b_<object>))))
-	     (if (null? res) #f res)))))
+  (let* ((raw-proc-hash
+	  (lambda lst-args
+	    (_i_call-proc proc-hash lst-args
+			   (list _b_<object> _b_<integer>))))
+	 (raw-proc-assoc
+	  (lambda lst-args
+	    (_i_call-proc proc-assoc lst-args
+			   ;; Maybe the second type should be more exact.
+			   (list _b_<object> _b_<object>)))))
     (hashx-remove! raw-proc-hash raw-proc-assoc hashtable obj-key)))
 
 (define (hash-count-elements ht)
   (hash-count (const #t) ht))
 
 
-(define (theme-object-assoc o al)
-  (let ((pr
-	 (find (lambda (pr) (eqv? o (car pr))) al)))
-    (if (eq? pr #f) '() pr)))
-
-
 (define (theme-string-assoc str al) 
   (assert (string? str))
-  (let ((pr
-	 (find (lambda (pr) (string=? str (car pr))) al)))
-    (if (eq? pr #f) '() pr)))
+  (assoc str al string=?))
 
 
 (define (theme-hash-for-each proc ht)
@@ -515,3 +605,44 @@
 	 (lambda l-args
 	   (_i_call-proc proc l-args (list _b_<object> _b_<object>)))))
     (hash-for-each pc-raw ht)))
+
+
+;; *** (standard-library hash-table2) ***
+
+(define hash-contents hash)
+
+(define theme-string-hash string-hash)
+
+
+
+;; *** (standard-library object-string-conversion) ***
+
+
+(define (theme-string->integer str)
+  (let ((nr (string->number str)))
+    (if (integer? nr)
+	nr
+	(raise (make-theme-d-condition 'string-to-integer-conv-error
+				       (list (cons 'str str)))))))
+
+
+(define (theme-string->real str)
+  (let ((nr (string->number str)))
+    (cond
+     ((integer? nr) (exact->inexact nr))
+     ((rational? nr) (exact->inexact nr))
+     ((and (real? nr) (not (exact? nr))) nr)
+     (else
+      (raise (make-theme-d-condition 'string-to-real-conv-error
+				     (list (cons 'str str))))))))
+
+
+(define (theme-string->real-number str)
+  (let ((nr (string->number str)))
+    (cond
+     ((integer? nr) nr)
+     ((and (real? nr) (not (exact? nr))) nr)
+     ((rational? nr) (cons (numerator nr) (denominator nr)))
+     (else
+      (raise (make-theme-d-condition 'string-to-real-conv-error
+				     (list (cons 'str str))))))))
