@@ -18,8 +18,8 @@
 (define is-type-fwd? '())
 
 
-;; Probably eq? could be used here.
-(define target-type=? eqv?)
+;; We had formerly eqv? here.
+(define target-type=? eq?)
 
 
 (define gl-ctr5 0)
@@ -224,6 +224,7 @@
 
 (define (make-field s-name to-type s-read-access s-write-access
 		    has-init-value? to-init-value)
+  (if (not (symbol? s-name)) (begin (display s-name) (newline)))
   (assert (symbol? s-name))
   ;;  (assert (is-target-object? to-type))
   (assert (is-access-spec? s-read-access))
@@ -422,7 +423,6 @@
 
 (define (make-target-class-var address module superclass fields inh? imm?
 			       ebv? ctr-access)
-  ;;  (dwl4 "make-target-class-var")
   (assert (is-address? address))
   (assert (is-module-name? module))
   (assert (is-entity? superclass))
@@ -453,7 +453,6 @@
 
 (define (make-builtin-target-class s-name superclass fields inh? imm?
 				   ebv? ctr-access)
-  ;;  (dwl4 "make-builtin-target-class")
   (assert (symbol? s-name))
   (assert (is-entity? superclass))
   (assert (list? fields))
@@ -523,7 +522,7 @@
 (define (make-empty-expression) empty-expression)
 
 
-(define tc-nil (create-primitive-class '<nil> #t #t #t #t '()))
+(define tc-nil (create-primitive-class '<null> #t #t #t #t '()))
 
 (define var-tc-nil (make-object-var tc-nil))
 
@@ -824,7 +823,6 @@
 	 inh? imm? ebv?
 	 instance-has-constructor?
 	 instance-ctr-access)
-  (dwl4 "make-parametrized-class-object0")
   (assert (string? name))
   (assert (boolean? inh?))
   (assert (boolean? imm?))
@@ -968,7 +966,6 @@
 
 
 (define (make-tt-union . types)
-  ;;  (dwl4 "make-tt-union")
   (make-target-object   
    tmt-union
    #t
@@ -1272,7 +1269,6 @@
 				always-returns?
 				never-returns?
 				static-method?)
-  (dwl4 "make-tpti-general-proc")
   (assert (boolean? simple?))
   (assert (is-target-object? arg-list-type))
   (assert (is-target-object? result-type))
@@ -1381,9 +1377,8 @@
 (define (make-gen-proc-object to-class name methods address)
   (assert (is-target-object? to-class))
   (assert (string? name))
-  ;; The following test is wrong. Methods are <target-object>'s.
   (assert (and (list? methods)
-	       (and-map? is-normal-variable? methods)))
+	       (and-map? is-target-object? methods)))
   (assert (or (null? address) (is-address? address)))
   (make-target-object 
    to-class
@@ -1660,7 +1655,7 @@
 
 
 (define (get-contents obj)
-  (assert (is-t-atomic-object? obj))
+  (assert (is-t-primitive-object? obj))
   (let ((contents (hfield-ref obj 'obj-prim-contents)))
     (if (eqv? contents to-nil) '() contents)))
 
